
/*
 *	Object for conference meeting communication
 *
 *	This object communicates with the wcs service to add,
 *	modify, end or start a conference. Logged in users can
 *	add, modify, delete and start conferences, other can only
 *	join using get or wait command.
 *
 *	Object responds to callback after successful queries.
 *
 *	Martin Ekblom 2012
 */

function wm_conference(oHandler){
	this.__handler = oHandler;

	this.__sid = null;
	this.__rid = null;
	this.__conference_id = null;

	this.__request = new cRequest('/wcs/');
	this.__participant_data = '';

	this._resource = 'WebClient';
};

wm_conference.xmlns = 'http://icewarp.com/groupware-api';

wm_conference.status = 'offline';

wm_conference.error = {
	101: 'MissingAccount',				// Majority of requests requires identification of the account who is doing the operation. If session is authenticated, this is taken from the authenticated account, otherwise it must be passed as a parameter
	102: 'InvalidMeetingInfo',			// Bad format of request
	103: 'NotAuthenticated',			// Command required authenticated session
	104: 'InvalidMeetingID',			// ID of non existing meeting used
	105: 'SaveError',					// Internal error during saving data to server database
	106: 'WaitFailure',					// Internal server error during wait operation
	107: 'WaitTimeout',					// Wait timeout, meeting was not started in the specified amount of time, another wait call can be performed
	108: 'CouldNotAuthenticate',		// Authentication failed
	109: 'InvalidSessionId',			// Non-existing session id used
	110: 'NotAccessMode',				// Authenticated user does not have access mode for meeting service
	111: 'NotEnabled',					// Meeting service is disabled (not started)
	112: 'ParseError',					// General error during parsing the request
	113: 'OnlyOrganizerCanDoThis',		// Start, Delete, Modify can be performed only by organizer and creator. If organizer is empty, it can be performed by everybody (authenticated)
	114: 'MikogoErrorResponse',			// Mikogo server refused to start a meeting, it returned informative xml, which is logged into meeting log
	115: 'MikogoStopError',				// Mikogo server refused to stop meeting
	116: 'EmptyMikogoResponse',			// Mikogo server sent empty response for start command
	117: 'MikogoInvalidXML',			// Mikogo server returned invalid XML as a response to start command
	118: 'RemoteIdNotSupported',		// Meeting id indicates that the meeting is not on local server but the performed operation can be done only on local server
	119: 'RemoteHTTPTimeout',			// Connection to remote server timed out before geting result
	120: 'RemoteCouldNotConnect',		// Could Not Connect to the remote server
	121: 'RemoteDisconnected',			// Remote server disconnected before sending all results
	122: 'RemoteGeneralError',			// General error in connection with comunication with remote server
	123: 'ListError',					// General error in listing meetings
	124: 'LicenseError',				// License does not allow to start more concurrent meetings
	125: 'NotLicensed',					// server does not have valid license for online meetings
	126: 'OnlySuperuserCanDoThis',		// some operations are available only for superuser. E.g. GetMeetingCount
	127: 'NotStarted',					// attempt to enable/disable recording or desktop session on non started meeting
	128: 'InvalidEventID',				// attempt to finish meeting recording with invalid parameters
	129: 'RecordingFileNotFound',		// attempt to finish meeting recording with invalid parameters (non existing file)
	130: 'SIPPipeError',				// could not communicate with VOIP server
	131: 'SIPNoParticipants',			// attempt to start recording of meeting without participants
	132: 'SpecialPermissionsRequired',	// attempt to to perform operation which only organizer or moderator can do
	133: 'OrganizerMissing',			// attempt to enable/disable recording of meeting without organizer
	134: 'RecordingGWerror',			// error in communication with GW service (check GW logs)
	135: 'NotSetUpProperly',			// VOIP server not probably configured - Either incorrect local port is used in VOIp settings, or conference account is not registered to SIP
	136: 'CouldNotStartDesktopSession',	// unknown error during starting desktop sharing session
	1000: 'NetworkRequestError'			// did not reach server, request failed due network issues on client
};

wm_conference.prototype.setId = function(cid) {
	this.__conference_id = cid;
}

/*
 * Login and functionality for organisers
 */

// log in for organiser
wm_conference.prototype.login = function() {
	this.authenticate();
}
// for manager pseudologin with callId as sessionId
wm_conference.prototype.manage = function(cid) {
	this.__sid = cid;
}
// log out manager or organiser
wm_conference.prototype.logout = function() {
	if(this.__sid) this.interact('logout');
}

// Create conference, implicitly logs in
wm_conference.prototype.create = function(oParams) {
	this.authenticate('create',oParams);
}

// Alter conferences
wm_conference.prototype.add = function(oParams) {
	var params = {
		type: 'conference',
		organizer: sPrimaryAccount
	};
	if(oParams) for(var p in oParams) params[p] = oParams[p];
	this.interact('add',params);
}
wm_conference.prototype.modify = function(oParams) {
	var params = {
		id: this.__conference_id,
		type: 'conference',
		organizer: sPrimaryAccount
	}
	if(oParams) for(var p in oParams) params[p] = oParams[p];
	this.interact('modify',params);
}
wm_conference.prototype.end = function() {
	this.interact('stop',{id: this.__conference_id});
}
wm_conference.prototype.erase = function() {
	this.interact('delete',{id: this.__conference_id});
}

// Start a conference
wm_conference.prototype.start = function() {
	this.interact('start',{id: this.__conference_id});
}

wm_conference.prototype.share = function(bShare) {
	if(bShare)
		this.interact('startsharing',{id: this.__conference_id});
	else
		this.interact('stopsharing',{id: this.__conference_id});
}

wm_conference.prototype.record = function(bRecord) {
	if(bRecord)
		this.interact('startrecording',{id: this.__conference_id});
	else
		this.interact('stoprecording',{id: this.__conference_id});
}

wm_conference.prototype.mute = function(usr,bMute) {
	this.interact('mute',{id: this.__conference_id, participant: usr, mute: bMute});
}

wm_conference.prototype.muteall = function(usr,bMute) {
	this.interact('muteall',{id: this.__conference_id, participant: usr, mute: bMute});
}

wm_conference.prototype.delegate = function(usr,bPermit) {
	if(bPermit==undefined) bPermit = true;
	this.interact('permit',{id: this.__conference_id, participant: usr, permit: bPermit});
}

wm_conference.prototype.dismiss = function(usr) {
	this.interact('dismiss',{id: this.__conference_id, participant: usr});
}

/*
 * Generally accessible functionality
 */

// Get general information about conference
wm_conference.prototype.get = function() {
	this.interact('get',{id: this.__conference_id});
}

// Get general information about conference but wait for start
wm_conference.prototype.wait = function(ms) {
	this.interact('wait',{timeout: ms || 60000});
}

// Check for changes to the conference
wm_conference.prototype.check = function() {
	this.interact('changed',{id: this.__conference_id});
}

// Continually check for changes in conference status
wm_conference.prototype.monitor = function() {
	var me = this;
	setTimeout(function(){
		if(dataSet.get('meeting',['live']))
			me.interact('monitor',{id: me.__conference_id});
	},1000);
}

/*
 * Application logic
 */

// Changing logon state of user (called after login/logout)
wm_conference.prototype.state = function(aResult,sType,oParams) {
	var r = aResult.Array.IQ[0];
	if(r.ATTRIBUTES['TYPE']=="error") {
		gui._create('alert', 'frm_alert', '', '', '', 'MAIN_MENU::CONFERENCE', 'CONFERENCE::NOTAUTHENTICATED');
	} else if(r.ATTRIBUTES['TYPE']=="result" && r.QUERY[0].DIGEST[0].VALUE) {
		this.__sid = r.QUERY[0].DIGEST[0].VALUE;
	}

	// Begin new conference if required
	if(dataSet.get('meeting',['live'])) {
		if(this.__handler)
			this.__handler.method.apply(this.__handler.object,[sType,r]);
	} else {
		if(this.__sid && !this.__conference_id && sType=='create')
			this.add(oParams);
		else if(this.__handler)
			this.__handler.method.apply(this.__handler.object,[sType,r]);
	}
}

wm_conference.prototype.response = function(aResult,sType) {
	var me = this;
	var r = this.retrieve(aResult);

	function decode(s) {
		if(s.length && s.indexOf('=')!=-1) {
			var tmp = s.split('&'), o = {};
			for(var i in tmp) {
				var n = tmp[i].split('=');
				if(n[0]) o[n[0]] = decodeURIComponent(n[1]);
			}
			return o;
		} else return {};
	}

	// If no error occured, add details to dataset
	if(!r.error) {
		if('id' in r)
			dataSet.add('meeting',['id'],r.id);

		if('desktop' in r && sType=='monitor')
			dataSet.add('meeting',['desktop'],parseInt(r.desktop) ? true : false)

		if(r.type=='conference')
			dataSet.add('meeting',['information'],{
				organizer: r.organizer,
				summary: r.summary,
				phone: r.information_pstn,
				sip: r.information_sip,
				url: r.information_url,
				invitation: r.information_body
			});

		if('recordinginprogress' in r)
			dataSet.add('meeting',['recording'],parseInt(r.recordinginprogress) ? true : false);

		if('mutestatus' in r)
			dataSet.add('meeting',['speaker'],parseInt(r.mutestatus) ? true : false);

		if(r.participants) {
			if(r.participants!=this.__participant_data) {
				// Save current participant data to detect changes next time
				this.__participant_data = r.participants;

				// Prepare data
				var ps = r.participants.split("\r\n"),
					participants = [],
					participating = [],
					roles = ['participant','manager','organizer'];

				// Add participants and their profile
				for(var i=0; i<ps.length; i++) {
					var p = decode(ps[i]);
					if(p.email) {
						var pdata = decodeURIComponent(p.participantdata).split('=');
						var silent = dataSet.get('meeting',['participants',p.participantid,'silent']);
						dataSet.add('meeting',['participants',p.participantid],{
							name:p.name,email:p.email,abilities:{
								muted: p.ismuted=='1' ? true : false,
								permission: p.haspermissions=='1' ? true : false,
								control: p.cancontrol=='1' ? true : false,
								client: pdata[0]=='client' && pdata[1]=='internal' ? true : false
							},
							role: roles[p.role],
							silent: silent===undefined || silent
						},true);
						participating.push(p.participantid);
					}
				}

				// Remove participants no more in conference
				for(var i in dataSet.get('meeting',['participants']))
					if(inArray(participating,i)==-1)
						dataSet.remove('meeting',['participants',i],true);

				// Update dataset
				dataSet.update('meeting',['participants']);
			}
		} else if('participants' in r)
			dataSet.remove('meeting',['participants']);

		if('started' in r)
			dataSet.add('meeting',['live'],parseInt(r.started) ? true : false);

	}

	// Handle response for and make callback
	if(r.error) {
		if(r.code==107 && sType=='wait')	{ // Waiting error, not really an error just response if conference still not started
			this.wait();
			return;
		}
		if(r.code==104 && sType=='changed' && dataSet.get('meeting',['live'])) { // If checking for changes and conference deleted, report a change
			dataSet.add('meeting',['live'],false);
			if(this.__handler)
				this.__handler.method.apply(this.__handler.object,[sType,{}]);
			return;
		} else {
			if(this.__handler)
				this.__handler.method.apply(this.__handler.object,['error',r]);
			return;
		}
	} else {
		switch(sType) {
			case 'add':
				this.__conference_id = r.id;
			case 'modify':

				break;
			case 'logout':
				sType='stop';
				break;
			case 'stop':
				if(this.__sid.indexOf('@')==-1) { // Only log out if we are not using pseudosession
					this.logout();
					return;
				}
				break;
			case 'delete':
				this.__conference_id = null;
				return; // No need for callback after delete
			case 'start':

				break;
			case 'wait':
			case 'get':

				break;
			case 'monitor':
				setTimeout(function(){
					me.monitor();
				},4000);
				return;
			case 'startrecording':
				if(r.result==1)
					dataSet.add('meeting',['recording'],true);
				break;
			case 'stoprecording':
				if(r.result==1)
					dataSet.add('meeting',['recording'],false);
				break;

			case 'startsharing':
			case 'stopsharing':
				break;

			case 'changed':
			default:
				return;
		}
	}

	if(this.__handler)
		this.__handler.method.apply(this.__handler.object,[sType,r]);

}

// Log in request
wm_conference.prototype.authenticate = function(sType,oParams) {
	var aRequest = {IQ: [{}]};
	aRequest.IQ[0].ATTRIBUTES = {
		SID: dataSet.get('main',['sid']),
		TYPE: 'get',
		FORMAT: 'xml'
	};
	aRequest.IQ[0].QUERY = [{METHOD: [{VALUE: 'meeting'}]}];
	aRequest.IQ[0].QUERY[0].ATTRIBUTES = {XMLNS: 'webmail:iq:auth'};

	request.sendArray(aRequest,[this,'state',[sType?sType:'login',oParams]]);
};

// Send general request
wm_conference.prototype.interact = function(sType,oParams) {
	if(oParams==undefined)
		oParams = {};

	var aRequest = {BODY: [{}]}, sCommand;
	aRequest.BODY[0].ATTRIBUTES = {
		XMLNS: wm_conference.xmlns,
		FROM: sPrimaryAccount
	};

	// Push all parameters except folder to array
	var aData = [];
	for(var i in oParams)
		if(i!='folder')
			aData.push(i+'='+encodeURIComponent(oParams[i]));

	if(this.__sid)
		aRequest.BODY[0].ATTRIBUTES.SID = this.__sid;

	aRequest.BODY[0].IQ = [{COMMAND: [{}],PARAMS: [{}]}];
	aRequest.BODY[0].IQ[0].ATTRIBUTES = {TYPE: 'request'};

	switch(sType) {
		case 'add':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'addmeetinginfo';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [
				{VALUE: this.__sid},
				{VALUE: aData.join('&')}
			];
			if(oParams.folder)
				aRequest.BODY[0].IQ[0].PARAMS[0].PARAM.push(
					{VALUE: ''}, // Not used
					{VALUE: 'folder='+encodeURIComponent(oParams.folder)}
				);
			break;
		case 'modify':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'addmeetinginfo';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [
				{VALUE: this.__sid},
				{VALUE: aData.join('&')},
				{VALUE: this.__conference_id}
			];
			break;
		case 'delete':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'deletemeeting';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: this.__sid},{VALUE: this.__conference_id}];
			break;
		case 'stop':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'stopmeeting';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: this.__sid},{VALUE: this.__conference_id}];
			break;
		case 'start':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'startmeeting';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: this.__sid},{VALUE: this.__conference_id},{VALUE: 'dontstartdesktop=1'}];
			break;
		case 'mute':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'managemeetingparticipant';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [
				{VALUE: this.__sid},
				{VALUE: this.__conference_id},
				{VALUE: oParams.participant},
				{VALUE: ['property=0','value='+(oParams.mute?1:0)].join('&')}
			];
			break;
		case 'muteall':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'managemeetingparticipant';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [
				{VALUE: this.__sid},
				{VALUE: this.__conference_id},
				{VALUE: oParams.participant},
				{VALUE: ['property=2','value='+(oParams.mute?1:0)].join('&')}
			];
			break;
		case 'permit':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'managemeetingparticipant';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [
				{VALUE: this.__sid},
				{VALUE: this.__conference_id},
				{VALUE: oParams.participant},
				{VALUE: ['property=1','value='+(oParams.permit?1:0)].join('&')}
			];
			break;
		case 'dismiss':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'managemeetingparticipant';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [
				{VALUE: this.__sid},
				{VALUE: this.__conference_id},
				{VALUE: oParams.participant},
				{VALUE: ['property=3','value=1'].join('&')}
			];
			break;
		case 'startsharing':
		case 'stopsharing':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'managemeetingdesktopsession';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [
				{VALUE: this.__sid},
				{VALUE: sPrimaryAccount},
				{VALUE: this.__conference_id},
				{VALUE: 'start='+(sType=='startsharing'?1:0)}
			];
			break;
		case 'startrecording':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'startmeetingrecording';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: this.__sid},{VALUE: this.__conference_id}];
			break;
		case 'stoprecording':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'stopmeetingrecording';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: this.__sid},{VALUE: this.__conference_id}];
			break;
		case 'get':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'getmeetinginfo';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: sPrimaryAccount},{VALUE: this.__conference_id}];
			break;
		case 'wait': // Participant waits for meeting to be started
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'waitmeeting';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: sPrimaryAccount},{VALUE: this.__conference_id},{VALUE: aData.timeout}];
			break;
		case 'changed': // Getting updated information about the meeting any time during the meeting
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'getmeetinginfo';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: sPrimaryAccount},{VALUE: this.__conference_id},{VALUE: 1}];
			break;
		case 'monitor': // Continously monitor the meeting information in interval
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'getmeetinginfo';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: sPrimaryAccount},{VALUE: this.__conference_id},{VALUE: 1}];
			break;
		case 'logout':
			aRequest.BODY[0].IQ[0].COMMAND[0].VALUE = 'logoutuser';
			aRequest.BODY[0].IQ[0].PARAMS[0].PARAM = [{VALUE: this.__sid}];
			break;
		default:
			return;	// Unsupported command
	}

	this.__request.sendArray(aRequest,[this,'response',[sType]]);
}

// Parse response data after request to object
wm_conference.prototype.retrieve = function(aResult) {
	var p = {}
	if(aResult && aResult.Array && aResult.Array.BODY) {
		var c = aResult.Array.BODY[0].IQ[0];
		if(c.ATTRIBUTES.TYPE=='result') {
			var v = c.VALUE[0].VALUE, vs;
			if(v.indexOf('&')!=-1) {
				vs = v.split('&');
				for(var i in vs) {
					var n = vs[i].split('=');
					if(n[0]) p[n[0]] = decodeURIComponent(n[1]);
				}
			} else {
				p.value = v;
			}
		} else {
			p.error = c.ERROR[0].VALUE;
			p.code = c.ERROR_CODE[0].VALUE;
		}
	} else {
		p.error = "Request error";
		p.code = 1000;
	}
	return p;
}

// Participant join from link or button
// (static method which can be used to join a meeting without login)
wm_conference.join = function(conferenceId) {
	if(conferenceId)
		gui._create('conference','frm_meeting','','',conferenceId);
}